#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oxpc_utils.h"

#include "oxpc_object.h"
#include "oxpc_invalid.h"

// runtime structure
typedef struct __attribute__((packed)) _oxpc_invalid {
  uint32_t type;
}* oxpc_invalid_t;

// runtime structure allocation
oxpc_object_t
oxpc_invalid_alloc(void)
{
  oxpc_invalid_t obj = NULL;
  obj = malloc(sizeof(*obj));
  if (!obj) {
    ERROR("unable to allocate memory for oxpc_invalid");
  }
  obj->type = OXPC_TYPE_INVALID;
  return (oxpc_object_t)obj;
}

// runtime structure free
void oxpc_invalid_free(oxpc_object_t obj)
{
  oxpc_check_type(obj, OXPC_TYPE_INVALID);
  free(obj);
}

// size of serialized structure
static size_t
oxpc_invalid_serialized_size(oxpc_object_t obj)
{
  oxpc_check_type(obj, OXPC_TYPE_INVALID);
  oxpc_invalid_t value = (oxpc_invalid_t)obj;
  return sizeof(*value);
}

// serialize runtime object to buffer
static void
oxpc_invalid_serialize_to_buffer(
                              oxpc_object_t obj,
                              void* buffer,
                              oxpc_port_list_t ports)
{
  oxpc_check_type(obj, OXPC_TYPE_INVALID);
  oxpc_invalid_t value = (oxpc_invalid_t)obj;
  memcpy(buffer, value, sizeof(*value));
}

struct oxpc_type_descriptor oxpc_invalid_type_descriptor = {
  .free = oxpc_invalid_free,
  .serialized_size = oxpc_invalid_serialized_size,
  .serialize_to_buffer = oxpc_invalid_serialize_to_buffer,
};

