#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "oxpc_utils.h"

#include "oxpc_object.h"
#include "oxpc_null.h"

// runtime structure
typedef struct __attribute__((packed)) _oxpc_null {
  uint32_t type;
}* oxpc_null_t;

// runtime structure allocation
oxpc_object_t
oxpc_null_alloc(void)
{
  oxpc_null_t obj = NULL;
  obj = malloc(sizeof(*obj));
  if (!obj) {
    ERROR("unable to allocate memory for oxpc_null");
  }
  obj->type = OXPC_TYPE_NULL;
  return (oxpc_object_t)obj;
}

// runtime structure free
void oxpc_null_free(oxpc_object_t obj)
{
  oxpc_check_type(obj, OXPC_TYPE_NULL);
  free(obj);
}

// size of serialized structure
static size_t
oxpc_null_serialized_size(oxpc_object_t obj)
{
  oxpc_check_type(obj, OXPC_TYPE_NULL);
  oxpc_null_t value = (oxpc_null_t)obj;
  return sizeof(*value);
}

// serialize runtime object to buffer
static void
oxpc_null_serialize_to_buffer(
                                oxpc_object_t obj,
                                void* buffer,
                                oxpc_port_list_t ports)
{
  oxpc_check_type(obj, OXPC_TYPE_NULL);
  oxpc_null_t value = (oxpc_null_t)obj;
  memcpy(buffer, value, sizeof(*value));
}

struct oxpc_type_descriptor oxpc_null_type_descriptor = {
  .free = oxpc_null_free,
  .serialized_size = oxpc_null_serialized_size,
  .serialize_to_buffer = oxpc_null_serialize_to_buffer,
};

